@@ -21,12 +21,7 @@
 #if MEDIAINFO_AES
     #include <aescpp.h>
 #endif //MEDIAINFO_AES
-#if MEDIAINFO_MD5
-    extern "C"
-    {
-        #include <md5.h>
-    }
-#endif //MEDIAINFO_MD5
+#include "MediaInfo/HashWrapper.h"
 //---------------------------------------------------------------------------
 
 namespace MediaInfoLib
@@ -103,6 +98,25 @@ public :
     #endif //MEDIAINFO_DEMUX
     Ztring  File_Name_WithoutDemux;
     bool   PTS_DTS_Needed;
+    enum ts_type
+    {
+        TS_NONE=0,
+        TS_PTS=1,
+        TS_DTS=2,
+        TS_ALL=TS_PTS|TS_DTS,
+    };
+    void   TS_Clear(ts_type Type=TS_ALL);
+    void   TS_Set(int64s Ticks, ts_type Type=TS_ALL);
+    void   TS_Set(File__Analyze* Parser, ts_type Type=TS_ALL);
+    void   TS_Add(int64s Ticks, ts_type Type=TS_ALL);
+    void   TS_Ajust(int64s Ticks);
+    int64s Frequency_c; //Frequency of the timestamp of the container (e.g. 90000 for MPEG-PS)
+    int64s Frequency_b; //Frequency of the timestamp of the bitstream (e.g. 48000 for AC-3)
+    #if MEDIAINFO_ADVANCED2
+    static const int64s NoTs=0x8000000000000000LL;
+    int64s PTSb; //In 1/Frequency_b if sub, in 1/(Frequency_c*Frequency_b) if in a container
+    int64s DTSb; //In 1/Frequency_b if sub, in 1/(Frequency_c*Frequency_b) if in a container
+    #endif //MEDIAINFO_ADVANCED2
     struct frame_info
     {
         int64u Buffer_Offset_End;
@@ -110,6 +124,11 @@ public :
         int64u PTS; //In nanoseconds
         int64u DTS; //In nanoseconds
         int64u DUR; //In nanoseconds
+        #if MEDIAINFO_ADVANCED2
+        int64s PTSc; //In 1/Frequency_c
+        int64s DTSc; //In 1/Frequency_c
+        int64u Frame_Count_AfterLastTimeStamp;
+        #endif //MEDIAINFO_ADVANCED2
 
         frame_info()
         {
@@ -118,6 +137,11 @@ public :
             PTS=(int64u)-1;
             DTS=(int64u)-1;
             DUR=(int64u)-1;
+            #if MEDIAINFO_ADVANCED2
+            PTSc=NoTs;
+            DTSc=NoTs;
+            Frame_Count_AfterLastTimeStamp=0;
+            #endif //MEDIAINFO_ADVANCED2
         }
     };
     frame_info FrameInfo;
@@ -147,6 +171,20 @@ public :
         };
         servicedescriptors* ServiceDescriptors;
     #endif
+    #if defined(MEDIAINFO_TELETEXT_YES)
+        struct teletext
+        {
+            std::map<std::string, Ztring>           Infos;
+            stream_t                                StreamKind;
+            size_t                                  StreamPos;
+
+            teletext()
+                : StreamKind(Stream_Max)
+                , StreamPos((size_t)-1)
+            {}
+        };
+        std::map<int16u, teletext>*                 Teletexts; //Key is teletext_magazine_number
+    #endif
 
     //Out
     int64u PTS_Begin;                  //In nanoseconds
@@ -286,7 +324,7 @@ protected :
     //Elements - Begin
     void Element_Begin ();
     void Element_Begin (const Ztring &Name);
-    inline void Element_Begin (const char *Name) {Element_Begin(Ztring().From_UTF8(Name));}
+    void Element_Begin (const char *Name);
     #define Element_Begin0() Element_Begin()
     #define Element_Begin1(_NAME) Element_Begin(_NAME)
     #define Element_Trace_Begin0() Element_Begin()
@@ -319,6 +357,7 @@ protected :
     #define Element_Info3(_A,_B,_C) Element_Info(_A, _B, _C)
     #define Element_Info1C(_CONDITION,_A) if (_CONDITION) Element_Info(_A)
     inline void Element_Info_From_Milliseconds (int64u Parameter)                  {if (Config_Trace_Level<1) return; Element_Info(Ztring().Duration_From_Milliseconds(Parameter));}
+    void Element_Parser (const Ztring &Name);
 
     //Elements - End
     inline void Element_End () {Element_End_Common_Flush();}
@@ -356,51 +395,51 @@ public :
 
     //Param - Main
     void Param      (const Ztring &Parameter, const Ztring& Value);
-    inline void Param      (const char*   Parameter, const Ztring& Value) {Param(Ztring().From_Local(Parameter), Value);};
-    inline void Param      (const char*   Parameter, const std::string& Value) {Param(Parameter, Ztring().From_Local(Value.c_str()));}
-    inline void Param      (const char*   Parameter, const char*   Value, size_t Value_Size=Unlimited, bool Utf8=true) {Param(Parameter, ToZtring(Value, Value_Size, Utf8));}
-    inline void Param      (const char*   Parameter, const int8u*  Value, size_t Value_Size=Unlimited, bool Utf8=true) {Param(Parameter, (const char*)Value, Value_Size, Utf8);}
-    inline void Param      (const char*   Parameter, bool   Value) {if (Value) Param(Parameter, "Yes"); else Param(Parameter, "No");}
-    inline void Param      (const char*   Parameter, int8u  Value) {Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring().From_CC1(Value)+__T(")"));}
-    inline void Param      (const char*   Parameter, int8s  Value) {Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring().From_CC1(Value)+__T(")"));}
-    inline void Param      (const char*   Parameter, int16u Value) {Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring().From_CC2(Value)+__T(")"));}
-    inline void Param      (const char*   Parameter, int16s Value) {Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring().From_CC2(Value)+__T(")"));}
-    inline void Param      (const char*   Parameter, int32u Value) {Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
-    inline void Param      (const char*   Parameter, int32s Value) {Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
-    inline void Param      (const char*   Parameter, int64u Value) {Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
-    inline void Param      (const char*   Parameter, int64s Value) {Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
-    inline void Param      (const char*   Parameter, int128u Value){Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
-    inline void Param_GUID (const char*   Parameter, int128u Value){Param(Parameter, Ztring().From_GUID(Value));}
-    inline void Param_UUID (const char*   Parameter, int128u Value){Param(Parameter, Ztring().From_UUID(Value));}
+    inline void Param      (const char*   Parameter, const Ztring& Value) {if (Trace_Activated) Param(Ztring().From_Local(Parameter), Value);};
+    inline void Param      (const char*   Parameter, const std::string& Value) { if (Trace_Activated) Param(Parameter, Ztring().From_Local(Value.c_str()));}
+    inline void Param      (const char*   Parameter, const char*   Value, size_t Value_Size=Unlimited, bool Utf8=true) {if (Trace_Activated) Param(Parameter, ToZtring(Value, Value_Size, Utf8));}
+    inline void Param      (const char*   Parameter, const int8u*  Value, size_t Value_Size=Unlimited, bool Utf8=true) {if (Trace_Activated) Param(Parameter, (const char*)Value, Value_Size, Utf8);}
+    inline void Param      (const char*   Parameter, bool   Value) {if (Trace_Activated) {if (Value) Param(Parameter, "Yes"); else Param(Parameter, "No");}}
+    inline void Param      (const char*   Parameter, int8u  Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring().From_CC1(Value)+__T(")"));}
+    inline void Param      (const char*   Parameter, int8s  Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring().From_CC1(Value)+__T(")"));}
+    inline void Param      (const char*   Parameter, int16u Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring().From_CC2(Value)+__T(")"));}
+    inline void Param      (const char*   Parameter, int16s Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring().From_CC2(Value)+__T(")"));}
+    inline void Param      (const char*   Parameter, int32u Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
+    inline void Param      (const char*   Parameter, int32s Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
+    inline void Param      (const char*   Parameter, int64u Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
+    inline void Param      (const char*   Parameter, int64s Value) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
+    inline void Param      (const char*   Parameter, int128u Value){if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value)+__T(" (0x")+Ztring::ToZtring(Value, 16)+__T(")"));}
+    inline void Param_GUID (const char*   Parameter, int128u Value){if (Trace_Activated) Param(Parameter, Ztring().From_GUID(Value));}
+    inline void Param_UUID (const char*   Parameter, int128u Value){if (Trace_Activated) Param(Parameter, Ztring().From_UUID(Value));}
     #ifdef SIZE_T_IS_LONG
-    inline void Param      (const char*   Parameter, size_t Value, intu Radix=16) {Param(Parameter, Ztring::ToZtring(Value, Radix).MakeUpperCase()+__T(" (")+Ztring::ToZtring(Value, 10).MakeUpperCase()+__T(")"));}
+    inline void Param      (const char*   Parameter, size_t Value, intu Radix=16) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value, Radix).MakeUpperCase()+__T(" (")+Ztring::ToZtring(Value, 10).MakeUpperCase()+__T(")"));}
     #endif //SIZE_T_IS_LONG
-    inline void Param      (const char*   Parameter, float32 Value, int8u AfterComma=3) {Param(Parameter, Ztring::ToZtring(Value, AfterComma));}
-    inline void Param      (const char*   Parameter, float64 Value, int8u AfterComma=3) {Param(Parameter, Ztring::ToZtring(Value, AfterComma));}
-    inline void Param      (const char*   Parameter, float80 Value, int8u AfterComma=3) {Param(Parameter, Ztring::ToZtring(Value, AfterComma));}
-    inline void Param      (const int32u  Parameter, const Ztring& Value) {Param(Ztring().From_CC4(Parameter), Value);};
-    inline void Param      (const int16u  Parameter, const Ztring& Value) {Param(Ztring().From_CC2(Parameter), Value);};
+    inline void Param      (const char*   Parameter, float32 Value, int8u AfterComma=3) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value, AfterComma));}
+    inline void Param      (const char*   Parameter, float64 Value, int8u AfterComma=3) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value, AfterComma));}
+    inline void Param      (const char*   Parameter, float80 Value, int8u AfterComma=3) {if (Trace_Activated) Param(Parameter, Ztring::ToZtring(Value, AfterComma));}
+    inline void Param      (const int32u  Parameter, const Ztring& Value) {if (Trace_Activated) Param(Ztring().From_CC4(Parameter), Value);};
+    inline void Param      (const int16u  Parameter, const Ztring& Value) {if (Trace_Activated) Param(Ztring().From_CC2(Parameter), Value);};
     #define Param1(_A) Param_(_A)
     #define Param2(_A,_B) Param(_A, _B)
     #define Param3(_A,_B,_C) Param(_A, _B, _C)
 
     //Param - Info
     void Param_Info (const Ztring &Parameter);
-    inline void Param_Info (const char*   Parameter) {Param_Info(Ztring().From_UTF8(Parameter));}
-    inline void Param_Info (const char*   Parameter, const char*   Measure)      {Param_Info(Ztring().From_UTF8(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int64u        Parameter, const char*   Measure=NULL) {Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int64s        Parameter, const char*   Measure=NULL) {Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int32u        Parameter, const char*   Measure=NULL) {Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int32s        Parameter, const char*   Measure=NULL) {Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int16u        Parameter, const char*   Measure=NULL) {Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int16s        Parameter, const char*   Measure=NULL) {Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int8u         Parameter, const char*   Measure=NULL) {Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (int8s         Parameter, const char*   Measure=NULL) {Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (float32       Parameter, int8u AfterComma=3, const char*   Measure=NULL) {Param_Info(Ztring::ToZtring(Parameter, AfterComma)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (float64       Parameter, int8u AfterComma=3, const char*   Measure=NULL) {Param_Info(Ztring::ToZtring(Parameter, AfterComma)+Ztring().From_UTF8(Measure));}
-    inline void Param_Info (float80       Parameter, int8u AfterComma=3, const char*   Measure=NULL) {Param_Info(Ztring::ToZtring(Parameter, AfterComma)+Ztring().From_UTF8(Measure));}
+    inline void Param_Info (const char*   Parameter) {if (Trace_Activated) Param_Info(Ztring().From_UTF8(Parameter));}
+    inline void Param_Info (const char*   Parameter, const char*   Measure)      {if (Trace_Activated) Param_Info(Ztring().From_UTF8(Parameter)+Ztring().From_UTF8(Measure));}
+    inline void Param_Info (int64u        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
+    inline void Param_Info (int64s        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
+    inline void Param_Info (int32u        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
+    inline void Param_Info (int32s        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
+    inline void Param_Info (int16u        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
+    inline void Param_Info (int16s        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
+    inline void Param_Info (int8u         Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
+    inline void Param_Info (int8s         Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
+    inline void Param_Info (float32       Parameter, int8u AfterComma=3, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter, AfterComma)+Ztring().From_UTF8(Measure));}
+    inline void Param_Info (float64       Parameter, int8u AfterComma=3, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter, AfterComma)+Ztring().From_UTF8(Measure));}
+    inline void Param_Info (float80       Parameter, int8u AfterComma=3, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter, AfterComma)+Ztring().From_UTF8(Measure));}
     #ifdef SIZE_T_IS_LONG
-    inline void Param_Info (size_t        Parameter, const char*   Measure=NULL) {Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
+    inline void Param_Info (size_t        Parameter, const char*   Measure=NULL) {if (Trace_Activated) Param_Info(Ztring::ToZtring(Parameter)+Ztring().From_UTF8(Measure));}
     #endif //SIZE_T_IS_LONG
     #define Param_Info1(_A) Param_Info(_A)
     #define Param_Info2(_A,_B) Param_Info(_A, _B)
@@ -408,7 +447,7 @@ public :
     #define Param_Info1C(_CONDITION,_A) if (_CONDITION) Param_Info(_A)
     #define Param_Info2C(_CONDITION,_A,_B) if (_CONDITION) Param_Info(_A, _B)
     #define Param_Info3C(_CONDITION,_A,_B,_C) if (_CONDITION) Param_Info(_A, _B, _C)
-    inline void Param_Info_From_Milliseconds (int64u Parameter)                  {Param_Info(Ztring().Duration_From_Milliseconds(Parameter));}
+    inline void Param_Info_From_Milliseconds (int64u Parameter)                  {if (Trace_Activated) Param_Info(Ztring().Duration_From_Milliseconds(Parameter));}
 
     //***************************************************************************
     // Information
@@ -962,7 +1001,7 @@ public :
     inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, int32s         Value, int8u Radix=10, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, Ztring::ToZtring(Value, Radix).MakeUpperCase(), Replace);}
     inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, int64u         Value, int8u Radix=10, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, Ztring::ToZtring(Value, Radix).MakeUpperCase(), Replace);}
     inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, int64s         Value, int8u Radix=10, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, Ztring::ToZtring(Value, Radix).MakeUpperCase(), Replace);}
-    inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, float32        Value, int8u AfterComma=3, bool Replace=false);
+    void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, float32        Value, int8u AfterComma=3, bool Replace=false);
     inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, float64        Value, int8u AfterComma=3, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, (float32)Value, AfterComma, Replace);}
     inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, float80        Value, int8u AfterComma=3, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, (float32)Value, AfterComma, Replace);}
     #ifdef SIZE_T_IS_LONG
@@ -1176,11 +1215,12 @@ private :
     size_t Data_Level;              //Current level for Data ("Top level")
 
     //Element
+public: //TO CHANGE
     BitStream_Fast* BS;             //For conversion from bytes to bitstream
     BitStream*      BT;             //For conversion from bytes to bitstream
 public : //TO CHANGE
     int64u Header_Size;             //Size of the header of the current element
-    const Ztring &Details_Get() {return Element[0].ToShow.Details;} //Direct access to details
+    Ztring &Details_Get(size_t Level=0) {return Element[Level].ToShow.Details;} //Direct access to details
     void   Details_Clear();
 protected :
     bool Trace_DoNotSave;
@@ -1318,11 +1358,11 @@ public :
         size_t      AES_Decrypted_Size;
     #endif //MEDIAINFO_AES
 
-    //MD5
-    #if MEDIAINFO_MD5
-        struct MD5Context*  MD5;
-        int64u              Md5_ParseUpTo;
-    #endif //MEDIAINFO_MD5
+    //Hash
+    #if MEDIAINFO_HASH
+        HashWrapper*        Hash;
+        int64u              Hash_ParseUpTo;
+    #endif //MEDIAINFO_HASH
 
     #if MEDIAINFO_SEEK
     private:
